@@ -27,7 +27,7 @@
 
 (require 'nrepl-dict)
 (require 'nrepl-client) ; `nrepl--mark-id-completed'
-(require 'cider-interaction)
+(require 'cider-eval)
 (require 'cider-client)
 (require 'cider-util)
 (require 'cider-inspector)
@@ -344,7 +344,7 @@ In order to work properly, this mode must be activated by
     ;; cider-nrepl has a chance to send the next message, and so that the user
     ;; doesn't accidentally hit `n' between two messages (thus editing the code).
     (when-let* ((proc (unless nrepl-ongoing-sync-request
-                        (get-buffer-process (cider-current-connection)))))
+                        (get-buffer-process (cider-current-repl)))))
       (accept-process-output proc 1))
     (unless cider--debug-mode
       (setq buffer-read-only nil)
@@ -602,7 +602,7 @@ is a coordinate measure in sexps."
                  code ns original-id
                  (if (and line column)
                      "you edited the code"
-                   "your tools.nrepl version is older than 0.2.11"))
+                   "your nREPL version is older than 0.2.11"))
                 (save-excursion
                   (cider--debug-move-point coor)
                   (point-marker)))))))))
