@@ -30,10 +30,18 @@
 
 ;;; Code:
 
-(require 'cider-interaction)
+(require 'cider-eval)
 (require 'clojure-mode)
 (require 'easymenu)
 
+(defcustom cider-scratch-initial-message
+  ";; This buffer is for Clojure experiments and evaluation.\n
+;; Press C-j to evaluate the last expression.\n\n"
+  "The initial message displayed in new scratch buffers."
+  :type 'string
+  :group 'cider
+  :package-version '(cider . "0.18.0"))
+
 (defvar cider-clojure-interaction-mode-map
   (let ((map (make-sparse-keymap)))
     (set-keymap-parent map clojure-mode-map)
@@ -44,11 +52,7 @@
       '("Clojure Interaction"
         (["Eval and print last sexp" #'cider-eval-print-last-sexp]
          "--"
-         ["Reset" #'cider-scratch-reset]
-         "--"
-         ["Set buffer connection" #'cider-assoc-buffer-with-connection]
-         ["Toggle buffer connection" #'cider-toggle-buffer-connection]
-         ["Reset buffer connection" #'cider-clear-buffer-local-connection])))
+         ["Reset" #'cider-scratch-reset])))
     map))
 
 (defconst cider-scratch-buffer-name "*cider-scratch*")
@@ -57,37 +61,37 @@
 (defun cider-scratch ()
   "Go to the scratch buffer named `cider-scratch-buffer-name'."
   (interactive)
-  (pop-to-buffer (cider-find-or-create-scratch-buffer)))
+  (pop-to-buffer (cider-scratch-find-or-create-buffer)))
 
-(defun cider-find-or-create-scratch-buffer ()
+(defun cider-scratch-find-or-create-buffer ()
   "Find or create the scratch buffer."
   (or (get-buffer cider-scratch-buffer-name)
-      (cider-create-scratch-buffer)))
+      (cider-scratch--create-buffer)))
 
 (define-derived-mode cider-clojure-interaction-mode clojure-mode "Clojure Interaction"
   "Major mode for typing and evaluating Clojure forms.
 Like clojure-mode except that \\[cider-eval-print-last-sexp] evals the Lisp expression
 before point, and prints its value into the buffer, advancing point.
 
-\\{cider-clojure-interaction-mode-map}")
+\\{cider-clojure-interaction-mode-map}"
+  (setq-local sesman-system 'CIDER))
 
-(defun cider--scratch-insert-welcome-message ()
+(defun cider-scratch--insert-welcome-message ()
   "Insert the welcome message for the scratch buffer."
-  (insert ";; This buffer is for Clojure experiments and evaluation.\n"
-          ";; Press C-j to evaluate the last expression.\n\n"))
+  (insert cider-scratch-initial-message))
 
-(defun cider-create-scratch-buffer ()
+(defun cider-scratch--create-buffer ()
   "Create a new scratch buffer."
   (with-current-buffer (get-buffer-create cider-scratch-buffer-name)
     (cider-clojure-interaction-mode)
-    (cider--scratch-insert-welcome-message)
+    (cider-scratch--insert-welcome-message)
     (current-buffer)))
 
 (defun cider-scratch-reset ()
   "Reset the current scratch buffer."
   (interactive)
   (erase-buffer)
-  (cider--scratch-insert-welcome-message))
+  (cider-scratch--insert-welcome-message))
 
 (provide 'cider-scratch)
 
