@@ -34,46 +34,73 @@
   (it "opens without error"
     (customize-group 'cider)))
 
-;;; connection browser
+(describe "cider-figwheel-main-init-form"
+  ;; whitespace checks sprinkled amongst other tests
+  (describe "from options"
+    (it "leaves keywords alone"
+      (let ((cider-figwheel-main-default-options ":dev "))
+        (expect (cider-figwheel-main-init-form) :to-equal "(do (require 'figwheel.main) (figwheel.main/start :dev))")))
+    (it "leaves maps alone"
+      (let ((cider-figwheel-main-default-options " {:a 1 :b 2}"))
+        (expect (cider-figwheel-main-init-form) :to-equal "(do (require 'figwheel.main) (figwheel.main/start {:a 1 :b 2}))")))
+    (it "leaves s-exprs alone"
+      (let ((cider-figwheel-main-default-options " (hashmap :a 1 :b 2)"))
+        (expect (cider-figwheel-main-init-form) :to-equal "(do (require 'figwheel.main) (figwheel.main/start (hashmap :a 1 :b 2)))")))
+    (it "prepends colon to plain names"
+      (let ((cider-figwheel-main-default-options " dev"))
+        (expect (cider-figwheel-main-init-form) :to-equal "(do (require 'figwheel.main) (figwheel.main/start :dev))"))))
 
-(describe "cider-connections-buffer"
-  (it "lists all the active connections"
-    (with-temp-buffer
-      (rename-buffer "*cider-repl test1*")
-      (let ((b1 (current-buffer)))
-        (setq-local nrepl-endpoint '("localhost" 4005))
-        (setq-local nrepl-project-dir "proj")
-        (setq-local cider-repl-type "clj")
-        (with-temp-buffer
-          (rename-buffer "*cider-repl test2*")
-          (let ((b2 (current-buffer)))
-            (setq-local nrepl-endpoint '("123.123.123.123" 4006))
-            (setq-local cider-repl-type "clj")
-            (let ((cider-connections (list b1 b2)))
-              (cider-connection-browser)
-              (with-current-buffer "*cider-connections*"
-                (expect (buffer-string) :to-equal "  REPL                           Host             Port    Project          Type
+  (describe "from minibuffer"
+    (before-each
+      ;; not necessary as of this writing, but it can't hurt
+      (setq-local cider-figwheel-main-default-options nil))
+    (it "leaves keywords alone"
+      (spy-on 'read-from-minibuffer :and-return-value " :prod ")
+      (expect (cider-figwheel-main-init-form) :to-equal "(do (require 'figwheel.main) (figwheel.main/start :prod))"))
+    (it "leaves maps alone"
+      (spy-on 'read-from-minibuffer :and-return-value " {:c 3 :d 4}")
+      (expect (cider-figwheel-main-init-form) :to-equal "(do (require 'figwheel.main) (figwheel.main/start {:c 3 :d 4}))"))
+    (it "leaves s-exprs alone"
+      (spy-on 'read-from-minibuffer :and-return-value "(keyword \"dev\") ")
+      (expect (cider-figwheel-main-init-form) :to-equal "(do (require 'figwheel.main) (figwheel.main/start (keyword \"dev\")))"))
+    (it "prepends colon to plain names"
+      (spy-on 'read-from-minibuffer :and-return-value "prod ")
+      (expect (cider-figwheel-main-init-form) :to-equal "(do (require 'figwheel.main) (figwheel.main/start :prod))"))))
 
-* *cider-repl test1*             localhost         4005   proj             Clojure
-  *cider-repl test2*             123.123.123.123   4006   -                Clojure\n\n")
+(describe "cider-project-type"
+  (describe "when there is a single project"
+    (it "returns that type"
+      (spy-on 'cider--identify-buildtools-present
+              :and-return-value '(lein))
+      (expect (cider-project-type) :to-equal 'lein)))
 
-                (goto-line 4)         ; somewhere in the second connection listed
-                (cider-connections-make-default)
-                (expect (car cider-connections) :to-equal b2)
-                (message "%s" (cider-connections))
-                (expect (buffer-string) :to-equal "  REPL                           Host             Port    Project          Type
+  (describe "when there are multiple possible project types"
+    (before-all
+      (spy-on 'cider--identify-buildtools-present
+              :and-return-value '(build-tool1 build-tool2))
+      ;; user choice build-tool2
+      (spy-on 'completing-read :and-return-value "build-tool2"))
 
-  *cider-repl test1*             localhost         4005   proj             Clojure
-* *cider-repl test2*             123.123.123.123   4006   -                Clojure\n\n")
-                (goto-line 4)         ; somewhere in the second connection listed
-                (cider-connections-close-connection)
-                (expect cider-connections :to-equal (list b1))
-                (expect (buffer-string) :to-equal "  REPL                           Host             Port    Project          Type
+    (it "returns the choice entered by user"
+      (expect (cider-project-type) :to-equal 'build-tool2))
 
-* *cider-repl test1*             localhost         4005   proj             Clojure\n\n")
-                (cider-connections-goto-connection)
-                (expect (current-buffer) :to-equal b1)
-                (kill-buffer "*cider-connections*")))))))))
+    (it "respects the value of `cider-preferred-build-tool'"
+      (let ((cider-preferred-build-tool 'build-tool1))
+        (expect (cider-project-type) :to-equal 'build-tool1))
+
+      (let ((cider-preferred-build-tool "invalid choice"))
+        (expect (cider-project-type) :to-equal 'build-tool2))
+
+      (let ((cider-preferred-build-tool 'build-tool3))
+        (expect (cider-project-type) :to-equal 'build-tool2))))
+
+  (describe "when there are no choices available"
+    (it "returns the value of `cider-jack-in-default'"
+      (spy-on 'cider--identify-buildtools-present
+              :and-return-value '())
+      (expect (cider-project-type) :to-equal cider-jack-in-default))))
+
+;;; cider-jack-in tests
 
 (describe "cider-inject-jack-in-dependencies"
   :var (cider-jack-in-dependencies cider-jack-in-nrepl-middlewares cider-jack-in-lein-plugins cider-jack-in-dependencies-exclusions)
@@ -86,25 +113,25 @@
       (setq-local cider-jack-in-dependencies-exclusions '()))
 
     (it "can inject dependencies in a lein project"
-      (expect (cider-inject-jack-in-dependencies "" "repl :headless" "lein")
+      (expect (cider-inject-jack-in-dependencies "" "repl :headless" 'lein)
               :to-equal "update-in :dependencies conj \\[org.clojure/tools.nrepl\\ \\\"0.2.12\\\"\\] -- update-in :plugins conj \\[cider/cider-nrepl\\ \\\"0.10.0-SNAPSHOT\\\"\\] -- repl :headless"))
 
     (it "can inject dependencies in a lein project with an exclusion"
-        (setq-local cider-jack-in-dependencies-exclusions '(("org.clojure/tools.nrepl" ("org.clojure/clojure"))))
-        (expect (cider-inject-jack-in-dependencies "" "repl :headless" "lein")
-                :to-equal "update-in :dependencies conj \\[org.clojure/tools.nrepl\\ \\\"0.2.12\\\"\\ \\:exclusions\\ \\[org.clojure/clojure\\]\\] -- update-in :plugins conj \\[cider/cider-nrepl\\ \\\"0.10.0-SNAPSHOT\\\"\\] -- repl :headless"))
+      (setq-local cider-jack-in-dependencies-exclusions '(("org.clojure/tools.nrepl" ("org.clojure/clojure"))))
+      (expect (cider-inject-jack-in-dependencies "" "repl :headless" 'lein)
+              :to-equal "update-in :dependencies conj \\[org.clojure/tools.nrepl\\ \\\"0.2.12\\\"\\ \\:exclusions\\ \\[org.clojure/clojure\\]\\] -- update-in :plugins conj \\[cider/cider-nrepl\\ \\\"0.10.0-SNAPSHOT\\\"\\] -- repl :headless"))
 
     (it "can inject dependencies in a lein project with multiple exclusions"
-        (setq-local cider-jack-in-dependencies-exclusions '(("org.clojure/tools.nrepl" ("org.clojure/clojure" "foo.bar/baz"))))
-        (expect (cider-inject-jack-in-dependencies "" "repl :headless" "lein")
-                :to-equal "update-in :dependencies conj \\[org.clojure/tools.nrepl\\ \\\"0.2.12\\\"\\ \\:exclusions\\ \\[org.clojure/clojure\\ foo.bar/baz\\]\\] -- update-in :plugins conj \\[cider/cider-nrepl\\ \\\"0.10.0-SNAPSHOT\\\"\\] -- repl :headless"))
+      (setq-local cider-jack-in-dependencies-exclusions '(("org.clojure/tools.nrepl" ("org.clojure/clojure" "foo.bar/baz"))))
+      (expect (cider-inject-jack-in-dependencies "" "repl :headless" 'lein)
+              :to-equal "update-in :dependencies conj \\[org.clojure/tools.nrepl\\ \\\"0.2.12\\\"\\ \\:exclusions\\ \\[org.clojure/clojure\\ foo.bar/baz\\]\\] -- update-in :plugins conj \\[cider/cider-nrepl\\ \\\"0.10.0-SNAPSHOT\\\"\\] -- repl :headless"))
 
     (it "can inject dependencies in a boot project"
-      (expect (cider-inject-jack-in-dependencies "" "repl -s wait" "boot")
+      (expect (cider-inject-jack-in-dependencies "" "repl -s wait" 'boot)
               :to-equal "-i \"(require 'cider.tasks)\" -d org.clojure/tools.nrepl\\:0.2.12 -d cider/cider-nrepl\\:0.10.0-SNAPSHOT cider.tasks/add-middleware -m cider.nrepl/cider-middleware repl -s wait"))
 
     (it "can inject dependencies in a gradle project"
-      (expect (cider-inject-jack-in-dependencies "" "--no-daemon clojureRepl" "gradle")
+      (expect (cider-inject-jack-in-dependencies "" "--no-daemon clojureRepl" 'gradle)
               :to-equal "--no-daemon clojureRepl")))
 
   (describe "when there are multiple dependencies"
@@ -113,11 +140,11 @@
       (setq-local cider-jack-in-nrepl-middlewares '("refactor-nrepl.middleware/wrap-refactor" "cider.nrepl/cider-middleware"))
       (setq-local cider-jack-in-dependencies-exclusions '()))
     (it "can inject dependencies in a lein project"
-      (expect (cider-inject-jack-in-dependencies "" "repl :headless" "lein")
+      (expect (cider-inject-jack-in-dependencies "" "repl :headless" 'lein)
               :to-equal "update-in :dependencies conj \\[org.clojure/tools.nrepl\\ \\\"0.2.12\\\"\\] -- update-in :plugins conj \\[refactor-nrepl\\ \\\"2.0.0\\\"\\] -- update-in :plugins conj \\[cider/cider-nrepl\\ \\\"0.11.0\\\"\\] -- repl :headless"))
 
     (it "can inject dependencies in a boot project"
-      (expect (cider-inject-jack-in-dependencies "" "repl -s wait" "boot")
+      (expect (cider-inject-jack-in-dependencies "" "repl -s wait" 'boot)
               :to-equal "-i \"(require 'cider.tasks)\" -d org.clojure/tools.nrepl\\:0.2.12 -d refactor-nrepl\\:2.0.0 -d cider/cider-nrepl\\:0.11.0 cider.tasks/add-middleware -m refactor-nrepl.middleware/wrap-refactor -m cider.nrepl/cider-middleware repl -s wait")))
 
   (describe "when there are global options"
@@ -127,14 +154,14 @@
       (setq-local cider-jack-in-lein-plugins '(("cider/cider-nrepl" "0.11.0")))
       (setq-local cider-jack-in-dependencies-exclusions '()))
     (it "can concat in a lein project"
-        (expect (cider-inject-jack-in-dependencies "-o -U" "repl :headless" "lein")
-                :to-equal "-o -U update-in :dependencies conj \\[org.clojure/tools.nrepl\\ \\\"0.2.12\\\"\\] -- update-in :plugins conj \\[cider/cider-nrepl\\ \\\"0.11.0\\\"\\] -- repl :headless"))
+      (expect (cider-inject-jack-in-dependencies "-o -U" "repl :headless" 'lein)
+              :to-equal "-o -U update-in :dependencies conj \\[org.clojure/tools.nrepl\\ \\\"0.2.12\\\"\\] -- update-in :plugins conj \\[cider/cider-nrepl\\ \\\"0.11.0\\\"\\] -- repl :headless"))
     (it "can concat in a boot project"
-        (expect (cider-inject-jack-in-dependencies "-C -o" "repl -s wait" "boot")
-                :to-equal "-C -o -i \"(require 'cider.tasks)\" -d org.clojure/tools.nrepl\\:0.2.12 -d cider/cider-nrepl\\:0.11.0 cider.tasks/add-middleware -m cider.nrepl/cider-middleware repl -s wait"))
+      (expect (cider-inject-jack-in-dependencies "-C -o" "repl -s wait" 'boot)
+              :to-equal "-C -o -i \"(require 'cider.tasks)\" -d org.clojure/tools.nrepl\\:0.2.12 -d cider/cider-nrepl\\:0.11.0 cider.tasks/add-middleware -m cider.nrepl/cider-middleware repl -s wait"))
     (it "can concat in a gradle project"
-        (expect (cider-inject-jack-in-dependencies "-m" "--no-daemon clojureRepl" "gradle")
-                :to-equal "-m --no-daemon clojureRepl")))
+      (expect (cider-inject-jack-in-dependencies "-m" "--no-daemon clojureRepl" 'gradle)
+              :to-equal "-m --no-daemon clojureRepl")))
 
   (describe "when there are predicates"
     :var (plugins-predicate middlewares-predicate)
@@ -185,10 +212,10 @@
               :and-return-value '(("refactor-nrepl" "2.0.0") ("cider/cider-nrepl" "0.11.0")))
       (setq-local cider-jack-in-dependencies-exclusions '()))
     (it "uses them in a lein project"
-      (expect (cider-inject-jack-in-dependencies "" "repl :headless" "lein")
+      (expect (cider-inject-jack-in-dependencies "" "repl :headless" 'lein)
               :to-equal "update-in :dependencies conj \\[org.clojure/tools.nrepl\\ \\\"0.2.12\\\"\\] -- update-in :plugins conj \\[refactor-nrepl\\ \\\"2.0.0\\\"\\] -- update-in :plugins conj \\[cider/cider-nrepl\\ \\\"0.11.0\\\"\\] -- repl :headless"))
     (it "uses them in a boot project"
-      (expect (cider-inject-jack-in-dependencies "" "repl -s wait" "boot")
+      (expect (cider-inject-jack-in-dependencies "" "repl -s wait" 'boot)
               :to-equal "-i \"(require 'cider.tasks)\" -d org.clojure/tools.nrepl\\:0.2.12 -d refactor-nrepl\\:2.0.0 -d cider/cider-nrepl\\:0.11.0 cider.tasks/add-middleware -m refactor-nrepl.middleware/wrap-refactor -m cider.nrepl/cider-middleware repl -s wait"))))
 
 (describe "cider-jack-in-auto-inject-clojure"
@@ -212,38 +239,18 @@
       (expect (cider-add-clojure-dependencies-maybe nil)
               :to-equal '(("Hello, I love you" "won't you tell me your name"))))))
 
-(describe "cider-project-type"
-  (describe "when there is a single project"
-    (it "returns that type"
-      (spy-on 'cider--identify-buildtools-present
-              :and-return-value '("lein"))
-      (expect (cider-project-type) :to-equal "lein")))
-
-  (describe "when there are multiple possible project types"
-    (before-all
-      (spy-on 'cider--identify-buildtools-present
-              :and-return-value '("build1" "build2"))
-      ;; user choice build2
-      (spy-on 'completing-read :and-return-value "build2"))
-
-    (it "returns the choice entered by user"
-      (expect (cider-project-type) :to-equal "build2"))
-
-    (it "respects the value of `cider-preferred-build-tool'"
-      (let ((cider-preferred-build-tool "build1"))
-        (expect (cider-project-type) :to-equal "build1"))
-
-      (let ((cider-preferred-build-tool "invalid choice"))
-        (expect (cider-project-type) :to-equal "build2"))
-
-      (let ((cider-preferred-build-tool "build3"))
-        (expect (cider-project-type) :to-equal "build2"))))
-
-  (describe "when there are no choices available"
-    (it "returns the value of `cider-default-repl-command'"
-      (spy-on 'cider--identify-buildtools-present
-              :and-return-value '())
-      (expect (cider-project-type) :to-equal cider-default-repl-command))))
+(describe "cider-normalize-cljs-init-options"
+  (describe "from options"
+    (it "leaves keywords alone"
+      (expect (cider-normalize-cljs-init-options ":dev") :to-equal ":dev"))
+    (it "leaves maps alone"
+      (expect (cider-normalize-cljs-init-options "{:a 1 :b 2}") :to-equal "{:a 1 :b 2}"))
+    (it "leaves s-exprs alone"
+      (expect (cider-normalize-cljs-init-options "(hashmap :a 1 :b 2)") :to-equal "(hashmap :a 1 :b 2)"))
+    (it "leaves vectors alone"
+      (expect (cider-normalize-cljs-init-options "[1 2 3]") :to-equal "[1 2 3]"))
+    (it "prepends colon to plain names"
+      (expect (cider-normalize-cljs-init-options "dev") :to-equal ":dev"))))
 
 (provide 'cider-tests)
 
